<?php


namespace Schaffen\Financial\IntegrationGcred\Repositories;

use DB;
use Freelabois\LaravelQuickstart\Extendables\Repository;

class IntegrationRepository extends Repository
{
    // protected $model = Route::class;
    // protected $searchableFields = [
    //     ['field' => 'rot_codigo'],
    //     ['field' => 'rot_descricao', 'operator' => 'ilike'],
    // ];
    public function lostContracts($filters){
        $per_page = 10;
        $page = null;
        $data_ini_pagamento = null;
        $data_fim_pagamento = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;

        extract($filters);

        $psi = DB::table('financeiro.pcg_sem_integracao')->pluck('psi_fk_pcg_codigo')->toArray();

       
        $sql = DB::table('financeiro.parcela')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original") ,
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'ppg_fk_par_codigo',
            'ppg_fk_pag_codigo',
            'ppg_valor_parcela',
            'ppg_juros_atraso',
            'ppg_multa',
            'ppg_juros_adicionais',
            'ppg_valor_pago',
            'par_data_alteracao',
            'pag_patrimonial'

        ])

        ->join('financeiro.parcela_pagamento', 'ppg_fk_par_codigo', 'par_codigo')
        ->join('financeiro.pagamento', 'pag_codigo', 'ppg_fk_pag_codigo')
        ->where('par_status', 2) // quitado
        ->whereIn('pag_patrimonial', $psi)

        ->when($data_ini_pagamento, function ($query) use ($data_ini_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date >= '%s'", $data_ini_pagamento));
        })
        ->when($data_fim_pagamento, function ($query) use ($data_fim_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date <= '%s'", $data_fim_pagamento));
        })
        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })
        ->paginate($per_page, ['*'], 'page', $page);

        return $sql;

    }

    public function installmentsData($filters){
        $per_page = 10;
        $page = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;
        $contrato_id = null;


        extract($filters);

        $sql = DB::table('financeiro.parcela')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original") ,
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'com_fk_con_codigo',
            'com_fk_mvf_codigo'
        ])
        ->join('contrato.contrato_mvf', 'par_fk_mvf_codigo', 'com_fk_mvf_codigo')
        ->where('par_status', 0) // aberto
        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })
        ->when($contrato_id, function ($query) use ($contrato_id) {
            return $query->where('com_fk_con_codigo', $contrato_id);
        })
        ->paginate($per_page, ['*'], 'page', $page);
        
        return $sql;

    }

    public function installmentsDataReceived($filters){
        $per_page = 10;
        $page = null;
        $data_ini_pagamento = null;
        $data_fim_pagamento = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;

        extract($filters);

        $sql = DB::table('financeiro.parcela')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original") ,
            DB::raw("to_char(pag_data_pagamento, 'dd/mm/yyyy') as pag_data_pagamento") ,
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'ppg_fk_par_codigo',
            'ppg_valor_parcela',
            'ppg_juros_atraso',
            'ppg_multa',
            'ppg_juros_adicionais',
            'ppg_valor_pago',
            'par_data_alteracao'
        ])

        ->join('financeiro.parcela_pagamento', 'ppg_fk_par_codigo', 'par_codigo')
        ->join('financeiro.pagamento', 'ppg_fk_pag_codigo', 'pag_codigo')
        ->where('par_status', '<>', 0) // Quitado / Parcialmente quitado
        
        ->when($data_ini_pagamento, function ($query) use ($data_ini_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date >= '%s'", $data_ini_pagamento));
        })
        ->when($data_fim_pagamento, function ($query) use ($data_fim_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date <= '%s'", $data_fim_pagamento));
        })

        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })
        ->paginate($per_page, ['*'], 'page', $page);
        
        return $sql;

    }

    public function personData($filters){
        $pes_codigo = null;
        extract($filters);

        $sql = DB::table('cadastro.pessoa')
        ->select([
            'pes_codigo',
            'pes_nome',
            'pes_fk_reg_codigo',
            'pes_fk_seg_codigo',
            'peco_telefone',
            'peco_codigo',
            'peco_contato',
            'peco_email',
            'peco_fk_pes_codigo',
            'peco_padrao',
            'pee_cep',
            'pee_numero',
            'pee_logradouro',
            'pee_bairro',
            'pee_cidade',
            'pee_uf',
            'pee_pais',           
            'pee_complemento',
            'pee_tipo_endereco',
            'pee_tipo_residencia',
            'pee_distancia_km',
            'pes_identificador',
            'pee_padrao'

        ])
        ->leftjoin('cadastro.pessoa_contato', 'peco_fk_pes_codigo', 'pes_codigo')
        ->leftjoin('cadastro.pessoa_endereco', 'pee_fk_pes_codigo', 'pes_codigo')
        ->where('pes_codigo', $pes_codigo)
        ->where('pee_padrao', 1)->get();
        return $sql;

    }

    public function contractData($filters){
    
        $per_page = 10;
        $page = null;
        $data_ini = null;
        $data_fim = null;

        extract($filters);

        $sql = DB::table('contrato.v$contrato', 'c')
        ->select([
            'c.con_codigo' , 
            DB::raw("to_char(c.con_data_aprovacao, 'dd/mm/yyyy') as con_data_aprovacao"),
            DB::raw("to_char(c.con_data_cancelamento, 'dd/mm/yyyy') as con_data_cancelamento"),
            DB::raw("to_char(c.con_data_vencimento, 'dd/mm/yyyy') as con_data_vencimento"),
            DB::raw("to_char(c.con_data_vencimento, 'dd') as con_dia_vencimento"),
            DB::raw("to_char(c.con_data_liberacao, 'dd/mm/yyyy') as con_data_liberacao"),
            DB::raw("to_char(c.con_data_liberacao, 'yyyy') as con_ano_liberacao"),
            'cm.com_fk_con_codigo',
            'cm.com_fk_mvf_codigo',
            'c.con_user_insert' ,
            DB::raw("to_char(c.con_date_insert, 'dd/mm/yyyy hh24:mi') as con_date_insert"),
            'c.con_user_update' ,
            DB::raw("to_char(c.con_date_update, 'dd/mm/yyyy hh24:mi') as con_date_update"),
            'c.con_fk_age_codigo' ,
            'c.con_fk_ban_codigo' ,
            DB::raw("con_numero_conta || '-' || con_digito_verificador || ' / ' || ban_descricao || '-'  || age_agencia || ' (' || age_municipio || '/' || age_uf || ')' as dados_conta") ,
            'c.con_fk_fil_codigo' ,
            'fil_descricao' ,
            'c.con_fk_pes_codigo' ,
            'cli.pes_nome' ,
            'cli.pes_identificador as pes_identificador_cliente' ,
            'cli.pes_tipo as pes_tipo_cliente',            
            'pe.pee_cep',
            'pe.pee_numero',
            'pe.pee_logradouro',
            'pe.pee_bairro',
            'pe.pee_cidade',
            'pe.pee_uf',
            'pe.pee_pais',           
            'pe.pee_complemento',
            'pe.pee_tipo_residencia',
            'pe.pee_distancia_km',
            'pc.peco_telefone',
            'pc.peco_codigo',
            'pc.peco_contato',
            'pc.peco_email',
            'cli.pes_identificador',
            'c.con_fk_pes_codigo_fun',
            DB::raw("(select f.pes_nome from cadastro.v\$pessoa f where f.pes_codigo = c.con_fk_pes_codigo_fun) as funcionario"),
            DB::raw("(select usu_codigo from acesso.v\$usuario where usu_nome = c.con_user_insert) as usu_codigo"),
            DB::raw("(select usu_codigo from acesso.v\$usuario where usu_nome = c.con_user_update) as usu_codigo_update"),
            'c.con_fk_pra_codigo',
            DB::raw("pra_quantidade || ' ' || (select cor_valor from public.v\$codigos_referencias where cor_codigo = 'CONTRATO.PRAZO' and cor_valor_referencia = pra_tipo) as descricao_prazo") ,
            'prom.pes_nome as promotora' ,
            'c.con_fk_pes_codigo_prom' ,
            'c.con_fk_pro_codigo' ,
            'pro_descricao' ,
            'pro_cheque' ,
            'pro_fk_gru_codigo' ,
            'c.con_fk_sta_codigo' ,
            'sta_descricao' ,
            'sta_cor' ,
            'sta_muda_fase' ,
            'c.con_fk_fas_codigo' ,
            'fas_descricao' ,
            'fas_tipo_fase' ,
            'c.con_numero_conta' ,
            'c.con_digito_verificador' ,
            'c.con_numero_vezes' ,
            'c.con_dias_carencia' ,
            'c.con_percentual_comissao_fun' ,
            'c.con_percentual_comissao_prod' ,
            'c.con_tipo' ,
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'CONTRATO.CON_TIPO' and cor_valor_referencia = con_tipo::text) as descricao_tipo") ,
            'c.con_tipo_pagamento' ,
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'CONTRATO.CON_TIPO_PAGAMENTO' and cor_valor_referencia = con_tipo_pagamento::text) as descricao_pagamento"),
            'pro_tipo_impressao' ,
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'PRODUTO.PRO_TIPO_IMPRESSAO' and cor_valor_referencia = pro_tipo_impressao::text) as descricao_tipo_impressao") ,
            'c.con_usuario_aprovacao' ,
            'c.con_usuario_cancelamento' ,
            'c.con_valor' ,
            'c.con_valor_comissao' ,
            'c.con_valor_total' ,
            'c.con_fase_autenticada' ,
            'c.con_coeficiente' ,
            'c.con_valor_parcela' ,
            'c.con_taxa_juro' ,
            'c.con_taxa_iof' ,
            'fas_exige_liberacao' ,
            DB::raw("con_numero_conta || '-' || con_digito_verificador || ' / ' || ban_descricao || ' - ' || age_agencia || ' (' || age_municipio || '/' || age_uf || ')' as dados_conta") ,
            DB::raw("fil_codigo_filial || '/' || to_char(con_data_digitacao, 'ddmm/yyyy')|| '/' || lpad(con_codigo::text, 6, '0') as codigo_contrato") ,
            DB::raw("to_char(c.con_data_digitacao, 'dd/mm/yyyy') as con_data_digitacao") ,
            'con_fk_pes_codigo_fiador' ,
            'con_fk_pes_codigo_fiador2' ,
            'fiador.pes_nome as pes_nome_fiador' ,
            'fiador2.pes_nome as pes_nome_fiador2' ,
            'c.con_nome_vendedor_veiculo',
            'c.con_cpf_vendedor_veiculo' ,
            'c.con_parecer_origem' ,
            'c.con_parecer_superior' ,
            'c.con_fk_mvf_codigo_comissao' ,
            'c.con_numero_beneficio' ,
            'c.con_fk_con_codigo' ,
            'c.con_valor_bem' ,
            'c.con_descricao_bem' ,
            'c.con_reneg_valor' ,
            DB::raw("(select con_codigo from contrato.v\$contrato c1 where c1.con_fk_con_codigo = c.con_codigo) as con_codigo_renegociacao"),
        ])


        ->join('cadastro.v$pessoa as cli', 'cli.pes_codigo', 'c.con_fk_pes_codigo')
        ->leftJoin('cadastro.v$pessoa_endereco as pe', function($join)
            {
                $join
                ->on('pe.pee_fk_pes_codigo', '=', 'cli.pes_codigo')
                ->on('pe.pee_padrao','=', DB::raw('1'));
            }
        )
        ->leftjoin('cadastro.v$pessoa_contato as pc', function($join)
            {
                $join
                ->on('pc.peco_fk_pes_codigo', '=', 'cli.pes_codigo')
                ->on('pc.peco_padrao','=', DB::raw('1'));
            }
        )
        ->leftJoin('cadastro.v$pessoa as fiador', 'fiador.pes_codigo', 'c.con_fk_pes_codigo_fiador')
        ->leftJoin('cadastro.v$pessoa as fiador2', 'fiador2.pes_codigo', 'c.con_fk_pes_codigo_fiador2')
        ->join('cadastro.v$filial', 'fil_codigo', 'c.con_fk_fil_codigo')
        ->join('contrato.v$status', 'sta_codigo', 'c.con_fk_sta_codigo')
        ->join('contrato.v$fase', 'fas_codigo', 'c.con_fk_fas_codigo')
        ->join('contrato.v$produto', 'pro_codigo', 'c.con_fk_pro_codigo')
        ->join('contrato.v$prazo', 'pra_codigo', 'c.con_fk_pra_codigo')
        ->join('cadastro.v$pessoa as prom', 'prom.pes_codigo', 'c.con_fk_pes_codigo_prom')
        ->join('contrato.contrato_mvf as cm', 'cm.com_fk_con_codigo', 'c.con_codigo')
        ->leftJoin('finan.v$banco', 'ban_codigo', 'c.con_fk_ban_codigo')
        ->leftJoin('finan.v$agencia', 'age_codigo', 'c.con_fk_age_codigo')
        ->when($data_ini, function ($query) use ($data_ini) {
            return $query->whereRaw(sprintf("c.con_data_digitacao::date >= '%s'", $data_ini));
        })
        ->when($data_fim, function ($query) use ($data_fim) {
            return $query->whereRaw(sprintf("c.con_data_digitacao::date <= '%s'", $data_fim));
        })
        ->paginate($per_page, ['*'], 'page', $page);

        
        return $sql;

    }

    protected function applyCustomFilters()
    {
        extract($this->filters);
    }

}